function l=tropism(tp,ap,pos,dx,s)
% getTropism:
%
% tp            tropism parameters (type,N,sigma), type: 0 plagio-, 1
%               gravi-, 2 exo-, 3 chemotropism; N: strength, sigma: deviation 
% ap            additional parameters
% pos           global position
% dx            axial resolution
% (s)           random stream (default: global stream)
%
% l             letter describing the change in root tip heading
%
% See also: tropismObjective
%
% Copyright 2008-2013 Daniel Leitner. See license.txt for details.
%

global geometry; % distance function

if nargin<5
    s = RandStream.getGlobalStream;
end

[alpha,beta] = tropism2(tp,ap,pos,dx,s); % get angular movement
d = geometry(testAlphaBeta(alpha,beta,pos,dx));

i=0;
dmin=inf;
while (d>0) % not valid
    i=i+1;
    j=0;
    while d>0 && j<5;
        beta = 2*pi*rand(s); % change beta
        d = geometry(testAlphaBeta(alpha,beta,pos,dx));
        if (d<dmin)
            dmin=d;
            bestA = alpha;
            bestB = beta;
        end
        j=j+1;
    end
    if d>0
        alpha=abs(alpha)+pi/40; % increase alpha
    end
    if i>20
        disp('Could not respect boundaries');
        alpha=bestA;
        beta=bestB;
        break
    end
end

l = letter('r',[alpha,beta]); % create letter



function x = testAlphaBeta(alpha,beta,pos,dx)
% testAlphaBeta: returns coordinates moving in new direction
%
% alpha         angle
% beta          angle
% dx            axial resolution
%
% x             postion after applying rotation and moving dx in that
%               direction
%

R = pos.R * [ 1 0 0; 0 cos(beta) -sin(beta); 0 sin(beta) cos(beta) ];  %roll;
R = R * [ cos(alpha) sin(alpha) 0; -sin(alpha) cos(alpha) 0; 0 0 1 ]; %turn;
x(1:3) = pos.x + dx*R(:,1);



function [alpha, beta] = tropism2(tp,ap,pos,dx,s)
% tropism: constrains the tropsim 
%
% tp            tropism paramters (type,N,sigma)
% pos           position
% ap            additional parameters
% dx            axial resolution
%
% alpha, beta   angles
%

if tp(1) == 3
    objfun = tropismObjective(tp(1),dx); % chemo, hydro, ...
else
    objfun = tropismObjective(tp(1),ap); % all others
end
N = tp(2);
sigma = tp(3);

alpha = sigma*randn(s)*sqrt(dx);
beta = rand(s)*2*pi;

% find optimal heading
if N>0
    N=N*sqrt(dx); % <- ?! 
    dn = N-floor(N); % for not integer N
    if dn>0
        if rand(s)<dn
            N=ceil(N);
        else
            N=floor(N);
        end
    end
    tt = alpha;
    rr = beta;
    zm = objfun(beta,alpha,pos);
    for i=1:N
        beta = rand(s)*2*pi;
        alpha = sigma*randn(s)*sqrt(dx);
        z=objfun(beta,alpha,pos);
        if (z<zm)
            zm=z;
            tt=alpha;
            rr=beta;
        end
    end
    alpha = tt;
    beta = rr;
end
