function s = writeString(str, mode)
% writeString: textual representation of the l-system string
%
% Parameters:
% string        list of l-system leters
% mode          'N' names of letters, 'S' length between branching points,
%               'T' age between branching points, 'P' positions of root
%               tips (default = 'N')
%
% s             :text describing the l-system string
%
% Copyright 2008-2013 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    mode = 'N';
end

names = ['F','f','C','#','[',']','+','-','&','^','\','/','|','r'];
s = [];
l = 0;
t=0;
first = 1;

i=1;
while i <= size(str,1)
    
    id = str(i,1);

    switch mode
        case 'N'
            if id<100
                s = [s, names(id)]; % static letter
            else
                s = [s, 'D']; % dynamic letter
            end
        case 'S'
            if id<100
                n = names(id);
                switch n
                    case 'F'
                        l=l+str(i,3); % length (cm)
                    case '[' %push
                        s = [s, sprintf(' %g [',l)];
                        l = 0;
                    case ']' %pop
                        s = [s, sprintf('%g]\n',l)];
                        l = 0;
                end
            end
        case 'T'
            if id<100
                n=names(id);
                switch n
                    case 'F'
                        t = str(i,4);
                        if first
                            s=[s, sprintf(' %g-',t)];
                            first=0;
                        end
                    case '[' %push
                        s=[s,sprintf('%g [',t)];
                        first = 1;
                    case ']' %pop
                        s=[s,sprintf('%g]\n',t)];
                        first = 1;
                end
            end
        case 'P'
            if id==300 % section growth
                l = str(i+1:i+3,:);
                p = struct('x',l(1,1:3)','C',l(2,1:3),'a',l(3,1),'t',l(3,2),...
                    'R',l(1:3,4:6));
                h = str(i+6,1:3);
                s=[s,sprintf('Point: [%g, %g, %g], Heading: [%g, %g, %g], Exo: [%g, %g, %g]\n',...
                    p.x(1),p.x(2),p.x(3),p.R(1,1),p.R(2,1),p.R(3,1),h(1),h(2),h(3))];
            end
    end
    
    i = i + str(i,2);
    
end